/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.planning.tactical;

import java.awt.Color;
import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.awt.image.BufferedImage;
import java.beans.IntrospectionException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import javax.imageio.ImageIO;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.model.AgroSenseBeanNode;
import nl.cloudfarming.client.model.PartField;
import org.netbeans.api.visual.model.ObjectState;
import org.openide.nodes.Children;
import org.openide.util.Exceptions;
import org.openide.util.lookup.Lookups;

/**
 * PartField node for data explorers
 *
 * @author Merijn Zengers
 */
public class PartFieldNode extends AgroSenseBeanNode<PartField> {

    public PartFieldNode(ProductionUnitLayer layer, ProductionUnitLayerObject layerObject, TacticalPlanningService service) throws IntrospectionException {
        super(layerObject.getObject(), Children.LEAF, Lookups.fixed(layerObject, layerObject.getObject(), service));
        setIconBaseWithExtension("nl/cloudfarming/client/icon/map-icon.png");
        layer.addPropertyChangeListener(Layer.PROP_PALETTE, new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                fireIconChange();
            }
        });
    }

    @Override
    public Transferable drag() throws IOException {
        return getBean();
    }

    @Override
    public Image getIcon(int type) {
        BufferedImage image = loadImage("map-icon.png");
        Layer layer = getLookup().lookup(Layer.class);
        Color baseColor = layer.getPalette().getBaseColor();
        int rgb = baseColor.getRGB();

        for (int x = 0; x < 15; x++) {
            image.setRGB(x, 0, rgb);
            //image.setRGB(x, 1, rgb);
            //image.setRGB(x, 14, rgb);
            image.setRGB(x, 15, rgb);
        }

        for (int y = 0; y < 15; y++) {
            image.setRGB(0, y, rgb);
            //image.setRGB(1, y, rgb);
            //image.setRGB(14, y, rgb);
            image.setRGB(15, y, rgb);
        }

        return image;
    }

    @Override
    public Image getOpenedIcon(int type) {
        return getIcon(type);
    }

    /**
     * Load the image based on the resourceName
     *
     * @param resourceName
     * @return BufferedImage the loaded image
     */
    private BufferedImage loadImage(final String resourceName) {
        try {
            BufferedImage image;
            final Class c = getClass();
            java.net.URL url;
            if ((url = c.getResource(resourceName)) == null) {
                return null;
            } else {
                image = ImageIO.read(url);
            }
            return image;
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
        }
        return null;
    }
}
