/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.tactical;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;
import nl.cloudfarming.client.geoviewer.GeographicalAction;
import nl.cloudfarming.client.geoviewer.Geometrical;
import nl.cloudfarming.client.model.PartField;
import nl.cloudfarming.client.util.GeometryUtil;

/**
 * Layer object that represents the specified field-object
 * 
 * @author Timon Veenstra
 */
public class ProductionUnitLayerObject implements Geometrical<MultiPolygon, PartField> {

    /**
     * Geometry of the underlying Field
     */
    private final MultiPolygon geometry;
    /**
     * The field this FieldLayerObject represents
     */
    private final PartField partField;

    public ProductionUnitLayerObject(final PartField field) {
        this.partField = field;
        geometry = (MultiPolygon) GeometryUtil.wktToGeometry(field.getWKT());
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getGeometry() 
     * @return The Geometry of type MultiPolygon
     */
    @Override
    public MultiPolygon getGeometry() {
        return this.geometry;
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getObject() 
     * @return The Object of type PartField this LayerObject is representing
     */
    @Override
    public PartField getObject() {
        return this.partField;
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getKey() 
     * @return Key as String
     */
    @Override
    public String getKey() {
        return Long.toString(this.partField.getId());
    }

    @Override
    public Long getKeyAttributeValue() {
        return this.partField.getId();
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getTooltipText() 
     * @return TooltipText as String
     */
    @Override
    public String getTooltipText() {
        return Long.toString(this.partField.getId()) + " " + this.partField.getName();
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getId() 
     * @return The Id of the wrapped object
     */
    @Override
    public long getId() {
        return this.partField.getId();
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getActions() 
     * @return Array with actions for this LayerObject
     */
    @Override
    public GeographicalAction[] getActions() {
        return null;
    }
    
    @Override
    public MultiPolygon getRenderObject(Envelope envelope) {
        if (envelope.intersects(geometry.getEnvelopeInternal())){
            return geometry;
        }
        return null;
    }

    @Override
    public Point getCentroid() {
        return this.geometry.getCentroid();
    }
}
