/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.tactical;

import java.awt.Color;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.beans.IntrospectionException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import javax.swing.AbstractAction;
import javax.swing.Action;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.AgroSenseBeanNode;
import nl.cloudfarming.client.model.ProductionUnit;
import nl.cloudfarming.client.model.Surface;
import nl.cloudfarming.client.planning.tactical.wizard.PlanningWizard;
import org.apache.commons.lang.StringUtils;
import org.openide.nodes.Children;
import org.openide.util.NbBundle;
import org.openide.util.lookup.Lookups;

/**
 *
 * @author Timon Veenstra
 */
public class ProductionUnitNode extends AgroSenseBeanNode<ProductionUnit> {

    private static final AppLogger LOG = AppLogFactory.getLogger(Surface.class);
    private ProductionUnit productionUnit;

    public ProductionUnitNode(final ProductionUnitLayer layer, ProductionUnit productionUnit, TacticalPlanningService service) throws IntrospectionException {
        super(productionUnit, Children.create(new PartFieldNodeFactory(productionUnit, layer), true), Lookups.fixed(productionUnit, layer,service));
        this.productionUnit = productionUnit;

        productionUnit.addPropertyChangeListener(ProductionUnit.PROP_CROP, new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                fireDisplayNameChange(getName(), getDisplayName());
            }
        });
        
        productionUnit.addPropertyChangeListener(ProductionUnit.PROP_COLOR, new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                layer.getPalette().setBaseColor((Color)evt.getNewValue());
                fireDisplayNameChange(getDisplayName(), getHtmlDisplayName());
            }
        });        

        this.productionUnit = productionUnit;
        setIconBaseWithExtension(Layer.LAYER_NODE_ICON_BASE);
    }

    @Override
    public String getHtmlDisplayName() {
        StringBuilder sb = new StringBuilder("<font color=\"#");
        Color color = (getBean().getColor()!=null)?getBean().getColor():Color.BLACK;
        sb.append(StringUtils.leftPad(Integer.toHexString(color.getRed()), 2, '0'));
        sb.append(StringUtils.leftPad(Integer.toHexString(color.getGreen()), 2, '0'));
        sb.append(StringUtils.leftPad(Integer.toHexString(color.getBlue()), 2, '0'));
        sb.append("\">");
        sb.append(getDisplayName());
        sb.append("</font>");
        return sb.toString();
    }
    
    @Override
    public String getDisplayName() {
        if (productionUnit.getRace() != null && productionUnit.getRace().getCrop() != null) {
            StringBuilder sb = new StringBuilder(productionUnit.getName());
            sb.append(" (");
            sb.append(productionUnit.getRace().getCrop());
            sb.append(")");
            return sb.toString();
        }else{
            return getName();
        }
    }

    @Override
    public Action[] getActions(boolean context) {
        if (productionUnit.getPartFields().size() > 0) {
            return new Action[]{new PlanningWizard()};
        }
        return null;

    }

    /**
     * Action to open the actionViewer window. In this window the actions which are planned on a ProductionUnit are displayed.
     */
    public class openActionViewerAction extends AbstractAction {

        public openActionViewerAction() {
            super(NbBundle.getMessage(openActionViewerAction.class, "actionviewertopcomponent.open.viewer.action.title"));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            ActionViewerTopcomponent actionViewerTopcomponent = ActionViewerTopcomponent.findInstance();
            if (null != actionViewerTopcomponent) {
                actionViewerTopcomponent.updateTable();
                actionViewerTopcomponent.open();
            }
        }
    }

    /**
     * Method overriden from super. When ProductionUnit is dragged the node should be expanded otherwise the partFields are nog loaded and therefore not shown on the map.
     * @return
     * @throws IOException 
     */
    @Override
    public Transferable drag() throws IOException {
        getChildren().getNodes();
        return super.drag();
    }
}
