/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.tactical.wizard.spray;

import java.util.List;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.model.CropProtection;
import nl.cloudfarming.client.planning.tactical.TacticalPlanningModule;
import org.openide.util.NbBundle;

/**
 * Panel which will present a list of Crop Protection Pesticides. A user can choose at least one or more and continue the wizard.
 * @author Merijn Zengers
 */
public class CropProtectionPanel extends JPanel {

    private static final int USAGE_DATE_COLUMN = 5;
    private static final String TABLEMODEL_COLUMN_ID = "ID";
    private static final String TABLEMODEL_COLUMN_AUTH_ID = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.crop_header_id");
    private static final String TABLEMODEL_COLUMN_PESTICIDE = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.crop_header_pesticide");
    private static final String TABLEMODEL_COLUMN_APPLICATION = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.crop_header_application");
    private static final String TABLEMODEL_COLUMN_PCT = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.crop_header_pct");
    private static final String TITLE = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.title");
    private JScrollPane pane = new JScrollPane();
    private JTable cropProtectionTable = new JTable();
    private DefaultTableModel availableTableModel;

    /** Creates new form PlanningVisualPanel2 */
    public CropProtectionPanel() {
        initComponents();
    }

    @Override
    public String getName() {
        return TITLE;
    }

    /** 
     * This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        setLayout(new MigLayout("wrap 1"));
        cropProtectionTable.setRowSelectionAllowed(true);

        pane.setViewportView(cropProtectionTable);
        this.add(pane);
    }

    public void setAvailableCropProtections(List<CropProtection> availableCropProtections) {

        if (!availableCropProtections.isEmpty()) {
            //Only if the table is empty
            if (availableTableModel == null) {
                String[][] displayList = new String[availableCropProtections.size()][USAGE_DATE_COLUMN];

                int i = 0;
                for (CropProtection cropProtection : availableCropProtections) {
                    
                    displayList[i][0] = String.valueOf(cropProtection.getId());
                    displayList[i][1] = String.valueOf(cropProtection.getAuthorizationNumber());
                    displayList[i][2] = cropProtection.getAgent().getName();
                    displayList[i][3] = cropProtection.getApplicationMethod();
                    int safetyTerm = cropProtection.getSafetyTerm();
                    displayList[i][4] = String.valueOf(safetyTerm);
                    
                    
                    i++;
                }

                availableTableModel = new DefaultTableModel(displayList,
                        new String[]{
                            TABLEMODEL_COLUMN_ID, TABLEMODEL_COLUMN_AUTH_ID, TABLEMODEL_COLUMN_PESTICIDE, TABLEMODEL_COLUMN_APPLICATION, TABLEMODEL_COLUMN_PCT
                        });
                
                cropProtectionTable.setModel(availableTableModel);
                
                // TODO: line below hides first (ID) column because cropprotections does not have a unique Auth ID at this moment.
                cropProtectionTable.getColumnModel().removeColumn(cropProtectionTable.getColumnModel().getColumn(0));
               }
        }
    }

    /**
     * Get the selected crop protection agents from the table.
     * @return String[rowIndex][colIndex] contains the selected cropProtections
     * rowIndex: The number of selected rows in the displayed table
     * colIndex: Fixed USAGE_DATE_COLUMN
     */
    public String[][] getSelectedCropProtections() {
        String[][] selectedValues = new String[cropProtectionTable.getSelectedRows().length][USAGE_DATE_COLUMN];
        int i = 0;
        for (int index : cropProtectionTable.getSelectedRows()) {
            selectedValues[i][0] = (String) availableTableModel.getValueAt(index, 0);
            selectedValues[i][1] = (String) availableTableModel.getValueAt(index, 1);
            selectedValues[i][2] = (String) availableTableModel.getValueAt(index, 2);
            selectedValues[i][3] = (String) availableTableModel.getValueAt(index, 3);
            selectedValues[i][4] = (String) availableTableModel.getValueAt(index, 4);
            i++;
        }
        return selectedValues;
    }
}